# -*- coding: utf-8 -*-
"""
@  time    : 2021/03/30
@  author  : XieYZ
"""
import jwt
from django.utils.encoding import smart_text

from rest_framework import exceptions
from rest_framework.authentication import BaseAuthentication, get_authorization_header
from rest_framework_jwt.settings import api_settings

from .user import user_info

jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
jwt_get_username_from_payload = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER


class BaseJSONWebTokenAuthentication(BaseAuthentication):
    """
    身份验证
    """
    JWT_AUTH_HEADER_PREFIX = "Bearer"
    JWT_AUTH_COOKIE = "Authorization"

    def authenticate(self, request):
        """
        验证TOKEN是否有效，如果有效，返回User和TOKEN，否则返回None
        """

        jwt_value = self.get_jwt_value(request)
        if jwt_value is None:
            raise exceptions.AuthenticationFailed()

        try:
            payload = jwt_decode_handler(jwt_value)
        except jwt.ExpiredSignature:
            msg = '凭证已过期'
            raise exceptions.AuthenticationFailed(msg)
        except jwt.DecodeError:
            msg = '无效的凭证'
            raise exceptions.AuthenticationFailed(msg)
        except jwt.InvalidTokenError:
            msg = '凭证已失效'
            raise exceptions.AuthenticationFailed(msg)
        user = self.authenticate_credentials(payload)
        return (user, jwt_value)

    def authenticate_credentials(self, payload):
        """
        Returns an active user that matches the payload's user id and email.
        """
        user = user_info(payload["user_id"])

        if not user:
            msg = '用户不存在'
            raise exceptions.NotFound(msg)

        email = user.get("email")
        if not email:
            msg = '无效的 payload.'
            raise exceptions.AuthenticationFailed(msg)

        is_active = user.get("is_active")
        if not is_active:
            msg = '账户已被禁用'
            raise exceptions.AuthenticationFailed(msg)

        return user


class Authentication(BaseJSONWebTokenAuthentication):
    www_authenticate_realm = 'api'

    def get_jwt_value(self, request):
        auth = get_authorization_header(request).split()
        auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()

        if not auth:
            if api_settings.JWT_AUTH_COOKIE:
                return request.COOKIES.get(api_settings.JWT_AUTH_COOKIE)
            return None
        if smart_text(auth[0].lower()) != auth_header_prefix:
            return None

        if len(auth) == 1:
            msg = 'Invalid Authorization header. No credentials provided.'
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = 'Invalid Authorization header. Credentials string should not contain spaces.'
            raise exceptions.AuthenticationFailed(msg)
        return auth[1]

    def authenticate_header(self, request):
        """
        Return a string to be used as the value of the `WWW-Authenticate`
        header in a `401 Unauthenticated` response, or `None` if the
        authentication scheme should return `403 Permission Denied` responses.
        """
        return '{0} realm="{1}"'.format(api_settings.JWT_AUTH_HEADER_PREFIX, self.www_authenticate_realm)



# -*- coding: utf-8 -*-
"""
@  time    : 2021/04/01
@  author  : XieYZ
"""
from django.core.cache import cache
from django.conf import settings

from jsonrpc.proxy import ServiceProxy

USER_INFO_KEY_PREFIX = "user_info"


def get_user_info_by_rpc(user_id):
    s = ServiceProxy(settings.AUTH_SERVICE_RPC_API)
    res = s.usercenter.get_user_info(user_id)
    if not res or not res.get("error"):
        result = res.get("result", None)
        cache.set(f"{USER_INFO_KEY_PREFIX}_{user_id}", result, timeout=3600)
        return result
    return None


def user_info(user_id):
    data = cache.get(f"{USER_INFO_KEY_PREFIX}_{user_id}")
    if not data:
        data = get_user_info_by_rpc(user_id)
    return data


def perm_check(user_id, perm):
    s = ServiceProxy(settings.AUTH_SERVICE_RPC_API)
    res = s.usercenter.permission_check(user_id, perm)
    if not res or not res.get("error"):
        return res.get("result", False)
    return False



# -*- coding: utf-8 -*-
"""
@  time    : 2021/03/30
@  author  : XieYZ
"""
from rest_framework import exceptions
from rest_framework.permissions import BasePermission

from .user import perm_check


class CheckActionPermMixin(BasePermission):

    authenticated_users_only = True
    message = '没有权限进行此操作！'

    def has_permission(self, request, view):
        method = request.method
        permissions_map = {
            "GET": "",
            "OPTIONS": "",
            "HEAD": "",
            "POST": "",
            "PUT": "",
            "PATCH": "",
            "DELETE": "",
        }
        if hasattr(view, 'permission_actions_map') and view.action in view.permission_actions_map and method in \
                view.permission_actions_map[view.action]:
            permissions_map.update(view.permission_actions_map[view.action])
        elif hasattr(view, 'permissions_map'):
            permissions_map.update(view.permissions_map)
        if not request.user and self.authenticated_users_only:
            return False
        if method in permissions_map:
            perm = permissions_map[method]
            if not perm:
                return True
            else:
                return perm_check(request.user["id"], perm)
        else:
            raise exceptions.MethodNotAllowed(method)



# !/usr/bin/env python
# -*- coding:utf-8 -*-

from setuptools import setup, find_packages

setup(
    name="django-troop-auth",
    version="0.1.2",
    url="https://github.com/troopstack/django-troop-auth.git",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=["django<=3.2.10", "djangorestframework", "djangorestframework-jwt", "django-json-rpc"]
)